
;**************************************************************************
;*                    INA VA Meter                                        *
;**************************************************************************
;**************************************************************************
;*          v 2.12 - 15.02.2015                                           *
;*                                                                        *
;*                                                                        *
;**************************************************************************
;*          Hardw.: PIC16F1823 used                                       *
;*          OSC.......: Internal osc. used     POWER.....:   5V extern.   *
;**************************************************************************



 INCLUDE "P16F1823.INC"


_U001			     EQU     B'00100110000100'		;see Page 48 of Datasheet

	;INT OSC w/o any output
	;MCLR disabled
	;WDT disabled
	;no protection
	;BOR disabled


_U002			     EQU     B'01100011111111'		;see Page 49 of Datasheet

	;write protection off
	;PLL enabled



	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002



;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"

v_ttt		EQU	H'22'
data_tmp	EQU	H'23'
c_tmp		EQU	H'24'
kbd_buffer	EQU	H'25'
v_tst1		EQU	H'26'
v_tst2		EQU	H'27'
v_tst3		EQU	H'28'
disp_pos	EQU	H'29'


v_ina_msb	EQU	H'2A'
v_ina_lsb	EQU	H'2B'

v_ina_v1	EQU	H'2C'
v_ina_v2	EQU	H'2D'

v_volt_1	EQU	H'2E'
v_volt_2	EQU	H'2F'
v_volt_3	EQU	H'30'
v_volt_4	EQU	H'31'
v_volt_5	EQU	H'32'

v_t004		EQU	H'33'

v_gv01		EQU	H'34'

v_sv_msb	EQU	H'35'
v_sv_lsb	EQU	H'36'

v_curr_1	EQU	H'37'
v_curr_2	EQU	H'38'
v_curr_3	EQU	H'39'
v_curr_4	EQU	H'3A'
v_curr_5	EQU	H'3B'
v_curr_sign	EQU	H'3C'

v_cnt01		EQU	H'3D'

v_tmp_msb	EQU	H'3E'
v_tmp_lsb	EQU	H'3F'

v_config1	EQU	H'40'
v_config2	EQU	H'41'

v_i2c_add	EQU	H'42'
v_temp1		EQU	H'43'


;--------------------------------------------------------------------------
;==========================================================================

		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.


;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;
;	       **********************************
;              **  RESET :  main boot routine  **
;              **********************************
;
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; initialization
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

init_main	nop
		clrf	BSR

		clrf	INTCON			;disable interrupts


		banksel	OPTION_REG	;BANK 1
		movlw	B'01111111'
		movwf	OPTION_REG	;see datasheet Page 176

		movlw	H'03'
		movwf	BSR		;select BANK 3
		clrf	ANSELA
		clrf	ANSELC

		movlw	H'01'
		movwf	BSR		;select BANK 1

		movlw	B'11101000'
		movwf	TRISA

		movlw	B'00001011'
		movwf	TRISC

		movf	OSCCON,0
		andlw	B'11101111'
		iorlw	B'01101000'
		movwf	OSCCON		;set 4MHz internal clock (default after reset is 500kHz)

		movlw	H'04'
		movwf	BSR		;select BANK 4
		clrf	WPUA		;disable weak pull-ups for PORTA
		clrf	WPUC		;disable weak pull-ups for PORTC

		bsf	WPUA,D'003'	;enable week pull-up for RA3
		bsf	WPUA,D'005'	;enable week pull-up for RA5
		bsf	WPUC,D'003'	;enable week pull-up for RC3

		banksel	LATC
		clrf	LATA
		clrf	LATC

		clrf	BSR		;select BANK 0


;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		call	i2c_init

		movlw	B'10010000'		;I2C address of the sensor
		movwf	v_i2c_add
		call	tc74_init

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

		call	disp_init

		btfsc	PORTA,D'003'
		goto	main_prg

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;display Firmware-release - but only if the output is switched off

		call	w_release

		call	dr2
		call	dr2
		call	dr2
		call	dr2
		call	dr2
		call	dr2
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Main Loop
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX


main_prg	nop

mm001		call	disp_clr1
		call	disp_clr2

;-----------------------------------
mm001_l		call	i2c_ina219_get		;get the value from INA219
		call	ina219_trans

		call	tc74_read		;read temperature from TA74

		call	w_ina219_1
;-----------------------------------
		call	d2			;just slow the loop down

		goto	mm001_l
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x               INA219  -  I2C communication routines                         x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;read bus voltage
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
in219_read02	call	i2c_start
		movlw	B'10000000'		;I2C address + write
		call	i2c_send
		movlw	D'002'			;pointer register to Bus Voltage Register
		call	i2c_send
		call	i2c_stop

;read BUS voltage
		call	i2c_start
		movlw	B'10000001'		;I2C address + read
		call	i2c_send

		call	i2c_receive
		movwf	v_ina_msb
		call	i2c_ack
;
		call	i2c_receive
		movwf	v_ina_lsb
		call	i2c_not_ack

		call	i2c_stop

		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;read shunt voltage
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
in219_read01	call	i2c_start
		movlw	B'10000000'		;I2C address + write
		call	i2c_send
		movlw	D'001'			;pointer register to Shunt Voltage Register
		call	i2c_send
		call	i2c_stop

;read SHUNT voltage
		call	i2c_start
		movlw	B'10000001'		;I2C address + read
		call	i2c_send

		call	i2c_receive
		movwf	v_sv_msb
		call	i2c_ack

		call	i2c_receive
		movwf	v_sv_lsb
		call	i2c_not_ack

		call	i2c_stop

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;read configuration register
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
in219_read00	call	i2c_start
		movlw	B'10000000'		;I2C address + write
		call	i2c_send
		movlw	D'000'			;pointer register to Control Register
		call	i2c_send
		call	i2c_stop

;read config register
		call	i2c_start
		movlw	B'10000001'		;I2C address + read
		call	i2c_send

		call	i2c_receive
		movwf	v_config1
		call	i2c_ack

		call	i2c_receive
		movwf	v_config2
		call	i2c_not_ack

		call	i2c_stop

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
i2c_ina219_get	call	in219_read00

		call	in219_read01
		call	in219_read02
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;translate received values into voltage information

ina219_trans	movf	v_config1,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	ina_no_values

		call	trans_bus_v
		call	trans_shn_v

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
ina_no_values	movlw	'-'
		movwf	v_curr_1
		movwf	v_curr_2
		movwf	v_curr_3
		movwf	v_curr_4

		movwf	v_volt_1
		movwf	v_volt_2
		movwf	v_volt_3
		movwf	v_volt_4

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
trans_shn_v	movlw	'0'
		movwf	v_curr_1
		movwf	v_curr_2
		movwf	v_curr_3
		movwf	v_curr_4
		movwf	v_curr_5
		movlw	'+'
		movwf	v_curr_sign

		clrf	v_cnt01

		btfsc	v_sv_msb,D'007'
		call	neg_curr				;there is (small) negative current

		movf	v_sv_msb,0
		movwf	v_ina_v1
		movf	v_sv_lsb,0
		movwf	v_ina_v2

;-----------------------------------------
		incf	v_ina_v1,1
		movf	v_ina_v2,0
		btfsc	STATUS,Z
		goto	xit56_5

		goto	xit56_2

xit56_1		movlw	D'000'
		movwf	v_ina_v2

xit56_2		call	inc_inai_m		;increment i
		call	inc_inai_m		;increment i

		decfsz  v_ina_v2,F
		goto	xit56_2
xit56_5		decfsz  v_ina_v1,F
		goto	xit56_1

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
neg_curr	movlw	'-'
		movwf	v_curr_sign

		movf	v_sv_msb,0
		movwf	v_tmp_msb
		movf	v_sv_lsb,0
		movwf	v_tmp_lsb

		clrf	v_sv_msb
		clrf	v_sv_lsb

nc_loop		incf	v_sv_lsb,1
		btfsc	STATUS,Z
		incf	v_sv_msb,1

		incf	v_tmp_lsb,1
		btfss	STATUS,Z
		goto	nc_loop

		incf	v_tmp_msb,1
		btfss	STATUS,Z
		goto	nc_loop

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
trans_bus_v	movf	v_ina_msb,0
		movwf	v_ina_v1
		movf	v_ina_lsb,0
		movwf	v_ina_v2

		rrf	v_ina_v1,1
		rrf	v_ina_v2,1
		rrf	v_ina_v1,1
		rrf	v_ina_v2,1
		rrf	v_ina_v1,1
		rrf	v_ina_v2,1
		movlw	B'00011111'
		andwf	v_ina_v1,1		;put the first three bits to '0'

		movlw	'0'
		movwf	v_volt_1
		movwf	v_volt_2
		movwf	v_volt_3
		movwf	v_volt_4
		movwf	v_volt_5


		incf	v_ina_v1,1
		movf	v_ina_v2,0
		btfsc	STATUS,Z
		goto	it56_5

		goto	it56_2

it56_1		movlw	D'000'
		movwf	v_ina_v2

it56_2		call	inc_inav_m		;increment V

		decfsz  v_ina_v2,F
		goto	it56_2
it56_5		decfsz  v_ina_v1,F
		goto	it56_1
;------------------------------------------------
;use space instead of very first '0' ------------
		movf	v_volt_1,0
		xorlw	'0'
		btfss	STATUS,Z
		goto	ina_t_ret
		movlw	' '
		movwf	v_volt_1
;------------------------------------------------
ina_t_ret	nop
		return
;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
;------------------------------------------------------------
inc_inav_m	nop
		movlw	D'004'			;LSB = 4mV
		movwf	v_t004
;
inc_n3		incf	v_volt_5,1
		movlw	':'
		xorwf	v_volt_5,0
		btfss	STATUS,Z
		goto	inc_n2
;
		movlw	'0'
		movwf	v_volt_5
		incf	v_volt_4,1
		movlw	':'
		xorwf	v_volt_4,0
		btfss	STATUS,Z
		goto	inc_n2
;
		movlw	'0'
		movwf	v_volt_4
		incf	v_volt_3,1
		movlw	':'
		xorwf	v_volt_3,0
		btfss	STATUS,Z
		goto	inc_n2
;
		movlw	'0'
		movwf	v_volt_3
		incf	v_volt_2,1
		movlw	':'
		xorwf	v_volt_2,0
		btfss	STATUS,Z
		goto	inc_n2
;
		movlw	'0'
		movwf	v_volt_2
		incf	v_volt_1,1
		movlw	':'
		xorwf	v_volt_1,0
		btfss	STATUS,Z
		goto	inc_n2
;
		movlw	'-'
		movwf	v_volt_1
		movwf	v_volt_2
		movwf	v_volt_3
		movwf	v_volt_4
		movwf	v_volt_5
		return

inc_n2		decfsz	v_t004,1
		goto	inc_n3

		return


;------------------------------------------------------------
;------------------------------------------------------------
inc_inai_m	nop
		movlw	D'001'			;LSB = 1mV
		movwf	v_t004
;
xinc_n3		incf	v_curr_5,1
		movlw	':'
		xorwf	v_curr_5,0
		btfss	STATUS,Z
		goto	xinc_n2
;
		movlw	'0'
		movwf	v_curr_5

		incf	v_curr_4,1
		movlw	':'
		xorwf	v_curr_4,0
		btfss	STATUS,Z
		goto	xinc_n2
;
		movlw	'0'
		movwf	v_curr_4
		incf	v_curr_3,1
		movlw	':'
		xorwf	v_curr_3,0
		btfss	STATUS,Z
		goto	xinc_n2
;
		movlw	'0'
		movwf	v_curr_3
		incf	v_curr_2,1
		movlw	':'
		xorwf	v_curr_2,0
		btfss	STATUS,Z
		goto	xinc_n2

		movlw	'0'
		movwf	v_curr_2
		incf	v_curr_1,1
		movlw	':'
		xorwf	v_curr_1,0
		btfss	STATUS,Z
		goto	xinc_n2
;
		movlw	'-'
		movwf	v_curr_1
		movwf	v_curr_2
		movwf	v_curr_3
		movwf	v_curr_4
		movwf	v_curr_5
		return

xinc_n2		decfsz	v_t004,1
		goto	xinc_n3

		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x                          INA219  -  display                                 x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
w_ina219_1	movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		btfss	PORTC,D'003'
		goto	w_ina219_2		;2-lines LCD

		movf	v_volt_1,0
		xorlw	'0'
		btfss	STATUS,Z
		goto	w_v1

		movlw	' '
		goto	w_v0

w_v1		movf	v_volt_1,0
w_v0		call	send_data

		movf	v_volt_2,0
		call	send_data

		movlw	','
		call	send_data

		movf	v_volt_3,0
		call	send_data

		btfss	PORTA,D'005'
		goto	w_ina219_1t

		movf	v_volt_4,0
		call	send_data

		call	wm_V

		call	wm_space
		call	wm_sls
;----------------------------------------------------------------------
;----------------------------------------------------------------------
		movlw	B'11000000'		;row: 1 / col: 8
		call	send_comm

;		movf	v_curr_sign,0
;		call	send_data

		call	wm_space

		btfss	PORTA,D'003'
		goto	w_off

		movf	v_curr_1,0
		call	send_data

		movlw	','
		call	send_data

		movf	v_curr_2,0
		call	send_data

		movf	v_curr_3,0
		call	send_data

		movf	v_curr_4,0
		call	send_data

		call	wm_A
		call	wm_space

		return
;----------------------------------------------------------------------
w_off		call	wm_O
		call	wm_F
		call	wm_F
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		btfsc	PORTC,D'003'
		return
;--------------------------------------
;2-lines LCD
		call	wm_space
		call	wm_space
		call	wm_space
		call	wm_space
		return
;----------------------------------------------------------------------
;----------------------------------------------------------------------
;----------------------------------------------------------------------
w_ina219_1t	call	wm_V
		call	wm_space

		call	wm_it_curr

		call	wm_space

		movf	v_temp1,0
		movwf	kbd_buffer

		xorlw	H'FF'
		btfsc	STATUS,Z
		goto	w_no_temp

		call	write_dec

		movlw	B'11011111'		;
		call	send_data

		return
;----------------------------------------------------------------------
w_no_temp	call	wm_minus
		call	wm_minus
		call	wm_space
		movlw	B'11011111'		;
		call	send_data

		return

;----------------------------------------------------------------------
wm_it_curr	btfss	PORTA,D'003'
		goto	w_off_t

		movf	v_curr_1,0
		call	send_data

		movlw	','
		call	send_data

		movlw	B'11000000'		;row: 1 / col: 8
		call	send_comm

		movf	v_curr_2,0
		call	send_data

		movf	v_curr_3,0
		call	send_data

		call	wm_A
		return
;----------------------------------------------------------------------
w_off_t		call	wm_O
		call	wm_F
		movlw	B'11000000'		;row: 1 / col: 8
		call	send_comm

		call	wm_F
		call	wm_space
		call	wm_space
		return


;----------------------------------------------------------------------
;----------------------------------------------------------------------
;----------------------------------------------------------------------
w_ina219_2	call	wm_V
		call	wm_space
		movlw	'='
		call	send_data
		call	wm_space

		movf	v_volt_1,0
		xorlw	'0'
		btfss	STATUS,Z
		goto	w2_v1

		movlw	' '
		goto	w2_v0

w2_v1		movf	v_volt_1,0
w2_v0		call	send_data

		movf	v_volt_2,0
		call	send_data

		movlw	','
		call	send_data

		movf	v_volt_3,0
		call	send_data

		movf	v_volt_4,0
		call	send_data

		call	wm_V


		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		btfss	PORTA,D'003'
		call	w_off

		btfsc	PORTA,D'003'
		call	w_i2_A

		movlw	B'11001011'		;row: 11 / col: 0
		call	send_comm

		movf	v_temp1,0
		movwf	kbd_buffer

		xorlw	H'FF'
		btfsc	STATUS,Z
		return				;no temperature value -- nothing to be displayed on 2-line LCD

		call	write_dec
		movlw	B'11011111'		;
		call	send_data
		call	wm_C
		return

;------------------------------------------------
w_i2_A		call	wm_I
		call	wm_space
		movlw	'='
		call	send_data
		call	wm_space

		movf	v_curr_1,0
		call	send_data

		movlw	','
		call	send_data

		movf	v_curr_2,0
		call	send_data

		movf	v_curr_3,0
		call	send_data

		movf	v_curr_4,0
		call	send_data

		call	wm_A
		call	wm_space

		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------------------------------------------------
d2c		MOVLW   D'090'
		MOVWF   TIMER2
cDEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

cDEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    cDEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    cDEL_LOOP1
		return
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr4		call	dr2
		call	dr2
		call	dr2
		return


;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;x           LCD Display Communication Routines                               x
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;RS  = RC4
;R/W = N/C
;E   = RC5

;D0 - D3 -- not connected

;D4  = RA0 -- RC0
;D5  = RA1 -- RC1
;D6  = RA2 -- RC2
;D7  = RA4 -- RC3


disp_init	nop

;1st step of init:
;		wait for LCD hardware reset (after power on) - no software reset performed...
		clrf	PORTC
		call	d2_p2

;-- SW Reset to be able to enter 4-bit mode
		bcf	PORTC,D'004'
		bcf	PORTC,D'005'		;EN = 0 => disable


		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTC,D'005'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTC,D'005'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

		movlw	B'00000011'
		call	set_LCD_D
		bsf	PORTC,D'005'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

		movlw	B'00000010'
		call	set_LCD_D
		bsf	PORTC,D'005'
		call	dy_comm
		bcf	PORTC,D'005'
		call	d2_p2

;-----------------------------------------------------------------
;2nd step of init:
;     command:           '001DNFxx' (SET FUNCTION)
;                            D = 1	--> 8 bit interface
;                            D = 0	--> 4 bit interface
;			 '001DNFxx'
;                             N = 1	--> 2 line display
;                             N = 0	--> 1 line display
;			 '001DNFxx'
;                              F = 1	--> 5x10
;			       F = 0	--> 5x7
;			 '001DNFxx'
		movlw	B'00101000'		;2 lines - (for disp 1x16: 1-8  = 00h - 0Fh
;                                                                          9-16 = 40h - 4Fh )
		call	send_comm
;-----------------------------------------------------------------
;3rd step of init:
;     command:           '00001DCB' (DISPLAY ON/OFF)
;                              D = 1	--> Display on
;			       D = 0	--> Display off
;           		 '00001DCB'
;                               C = 1	--> Cursor on
;				C = 0	--> Cursor off
;           		 '00001DCB'
;				 B = 1	--> Blinking on
;				 B = 0	--> Blinking off
		movlw	B'00001000'
		call	send_comm

		movlw	B'00001111'
		movlw	B'00001100'
		call	send_comm
;-----------------------------------------------------------------
;4th step of init:
;     command:           '000001DS' (ENTRY MODE SET)
;				D = 1	--> Increase
;				D = 0	--> Decrease
;		         '000001DS'
;				 S = 1	--> display is shifted
;				 S = 0	--> display is not shifted
;		         '000001DS'
		movlw	B'00000110'
		call	send_comm
;-----------------------------------------------------------------
;clear the display
		movlw	B'00000001'
		call	send_comm
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------

set_LCD_D	nop
		movwf	v_ttt

		btfss	v_ttt,D'000'
		bcf	PORTA,D'000'
		call	set_0

		btfss	v_ttt,D'001'
		bcf	PORTA,D'001'
		call	set_1

		btfss	v_ttt,D'002'
		bcf	PORTA,D'002'
		call	set_2

		btfss	v_ttt,D'003'
		bcf	PORTA,D'004'
		call	set_3
		return
;-------------------------------------------
set_0		btfsc	v_ttt,D'000'
		bsf	PORTA,D'000'
		return

set_1		btfsc	v_ttt,D'001'
		bsf	PORTA,D'001'
		return

set_2		btfsc	v_ttt,D'002'
		bsf	PORTA,D'002'
		return

set_3		btfsc	v_ttt,D'003'
		bsf	PORTA,D'004'
		return
;------------------------------------------------------------------------------------------------
;8-bit data transfer routines
;------------------------------------------------------------------------------------------------
send_data	bcf	PORTC,D'005'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;higher 4 bits are put to RA0-RA3
		call	scbd			;send

		movf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;lower 4 bits are put to RA0-RA3
		call	scbd			;send

		return

;set control bits
scbd		bsf	PORTC,D'004'		;RS -> 1 (data)
		bsf	PORTC,D'005'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTC,D'005'		;EN -> 0
		call	d1			;wait a little bit...

		return
;------------------------------------------------------------------------------------------------
send_comm	bcf	PORTC,D'005'		;EN -> 0
		movwf	data_tmp

		swapf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;higher 4 bits are put to RA0-RA3
		call	scbc			;send

		movf	data_tmp,0
		andlw	B'00001111'
		call	set_LCD_D		;lower 4 bits are put to RA0-RA3
		call	scbc			;send

		return

;set control bits
scbc		bcf	PORTC,D'004'		;RS -> 0 (command)
		bsf	PORTC,D'005'		;EN -> 1
		call	dy_comm			;wait a while...
		bcf	PORTC,D'005'		;EN -> 0
		call	d1			;wait a little bit...
		return
;-----------------------------------------------------------------------------------------------
;----------------------------------------------
dy_comm		call	dy_comm2	;01
		call	dy_comm2	;02
		call	dy_comm2	;03
		return

dy_comm2	MOVLW   D'010'			;delay value changed: original value was: 010
		MOVWF   TIMER2
DEL_LOOP22c	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP22c
		return
;---------------------------------------------------------------------------
;
d1		call	d1b		;01
		call	d1b		;02
		call	d1b		;02
		call	d1b		;02
		return

d1b		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP11	DECFSZ  TIMER2,F
                GOTO    DEL_LOOP11
		return
;
d2_p2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOPx1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOPx2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPx2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPx1
		return

;---------------------------------------------------------------------------
;---------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
clear_disp	call	disp_clr1
		call	disp_clr2
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
n9_char		movlw	B'11000000'		;od znaku c.9 treba zmenit adresu na 40h
		call	send_comm

		movlw	B'10000000'
		movwf	disp_pos
		return
;------------------------------------------------------------------------------------------------
;------------------------------------------------------------------------------------------------
init2		movlw	B'00000001'
;		call	send_comm
		movlw	B'10000000'
		call	send_comm
		return

;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
;character set
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

wm_space	movlw	' '
		call	send_data
		return

wm_dvojbodka	movlw	':'
		call	send_data
		return

wm_point	nop
wm_bodka	movlw	'.'
		call	send_data
		return

wm_hviezdicka	movlw	'*'
		call	send_data
		return

wm_otaznik	movlw	'?'
		call	send_data
		return

wm_sls		movlw	'/'
		call	send_data
		return

wm_kk		movlw	'#'
		call	send_data
		return

wm_minus	movlw	'-'
		call	send_data
		return

wm_plus		movlw	'+'
		call	send_data
		return

wm_hvizdicka	movlw	'*'
		call	send_data
		return

wm_zat1		movlw	'('
		call	send_data
		return

wm_zat2		movlw	')'
		call	send_data
		return

wm_sip1		movlw	'>'
		call	send_data
		return

wm_sip2		movlw	'<'
		call	send_data
		return

wm_stup		movlw	H'DF'
		call	send_data
		return

wm_rs		movlw	'='
		call	send_data
		return

wm_and		movlw	'&'
		call	send_data
		return
;---------- cislice ----------------------------------------
wm_0		movlw	'0'
		call	send_data
		return

wm_1		movlw	'1'
		call	send_data
		return

wm_2		movlw	'2'
		call	send_data
		return

wm_3		movlw	'3'
		call	send_data
		return

wm_4		movlw	'4'
		call	send_data
		return

wm_5		movlw	'5'
		call	send_data
		return

wm_6		movlw	'6'
		call	send_data
		return

wm_7		movlw	'7'
		call	send_data
		return

wm_8		movlw	'8'
		call	send_data
		return

wm_9		movlw	'9'
		call	send_data
		return

;--------- velke pismena -----------------------------------
wm_A		movlw	'A'
		call	send_data
		return

wm_B		movlw	'B'
		call	send_data
		return

wm_C		movlw	'C'
		call	send_data
		return

wm_D		movlw	'D'
		call	send_data
		return

wm_E		movlw	'E'
		call	send_data
		return

wm_F		movlw	'F'
		call	send_data
		return

wm_G		movlw	'G'
		call	send_data
		return

wm_H		movlw	'H'
		call	send_data
		return

wm_I		movlw	'I'
		call	send_data
		return

wm_J		movlw	'J'
		call	send_data
		return

wm_K		movlw	'K'
		call	send_data
		return

wm_L		movlw	'L'
		call	send_data
		return

wm_M		movlw	'M'
		call	send_data
		return

wm_N		movlw	'N'
		call	send_data
		return

wm_O		movlw	'O'
		call	send_data
		return

wm_P		movlw	'P'
		call	send_data
		return

wm_Q		movlw	'Q'
		call	send_data
		return

wm_R		movlw	'R'
		call	send_data
		return

wm_S		movlw	'S'
		call	send_data
		return

wm_T		movlw	'T'
		call	send_data
		return

wm_U		movlw	'U'
		call	send_data
		return

wm_V		movlw	'V'
		call	send_data
		return

wm_W		movlw	'W'
		call	send_data
		return

wm_X		movlw	'X'
		call	send_data
		return

wm_Y		movlw	'Y'
		call	send_data
		return

wm_Z		movlw	'Z'
		call	send_data
		return

;--------- male pismena -----------------------------------

wm_a		movlw	'a'
		call	send_data
		return

wm_b		movlw	'b'
		call	send_data
		return

wm_c		movlw	'c'
		call	send_data
		return

wm_d		movlw	'd'
		call	send_data
		return

wm_e		movlw	'e'
		call	send_data
		return

wm_f		movlw	'f'
		call	send_data
		return

wm_g		movlw	'g'
		call	send_data
		return

wm_h		movlw	'h'
		call	send_data
		return

wm_i		movlw	'i'
		call	send_data
		return

wm_k		movlw	'k'
		call	send_data
		return

wm_l		movlw	'l'
		call	send_data
		return

wm_m		movlw	'm'
		call	send_data
		return

wm_n		movlw	'n'
		call	send_data
		return

wm_o		movlw	'o'
		call	send_data
		return

wm_p		movlw	'p'
		call	send_data
		return

wm_r		movlw	'r'
		call	send_data
		return

wm_s		movlw	's'
		call	send_data
		return

wm_t		movlw	't'
		call	send_data
		return

wm_u		movlw	'u'
		call	send_data
		return

wm_v		movlw	'v'
		call	send_data
		return

wm_w		movlw	'w'
		call	send_data
		return

wm_x		movlw	'x'
		call	send_data
		return

wm_y		movlw	'y'
		call	send_data
		return
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


;---------------------------------------------------------------------------
disp_clr1	movlw	B'10000000'		;line 1 / char0
		call	send_comm

disp_clrx	call	wm_space	;01
		call	wm_space	;02
		call	wm_space	;03
		call	wm_space	;04
		call	wm_space	;05
		call	wm_space	;06
		call	wm_space	;07
		call	wm_space	;08
		call	wm_space	;09
		call	wm_space	;10
		call	wm_space	;11
		call	wm_space	;12
		call	wm_space	;13
		call	wm_space	;14
		call	wm_space	;15
		call	wm_space	;16
		call	wm_space	;17
		call	wm_space	;18
		call	wm_space	;19
		call	wm_space	;20
		call	wm_space	;21
		call	wm_space	;22
		call	wm_space	;23
		call	wm_space	;24

		return
;---------------------------------------------------------------------------
disp_clr2	movlw	B'11000000'		;line 2 / char0
		call	send_comm

		goto	disp_clrx

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
convert2ascii	movwf	c_tmp
;
		xorlw	H'15'
		btfsc	STATUS,Z
		goto	wm_Q
;
		movf	c_tmp,0
		xorlw	H'1D'
		btfsc	STATUS,Z
		goto	wm_W
;
		movf	c_tmp,0
		xorlw	H'24'
		btfsc	STATUS,Z
		goto	wm_E
;
		movf	c_tmp,0
		xorlw	H'2D'
		btfsc	STATUS,Z
		goto	wm_R
;
		movf	c_tmp,0
		xorlw	H'2C'
		btfsc	STATUS,Z
		goto	wm_T
;
		movf	c_tmp,0
		xorlw	H'35'
		btfsc	STATUS,Z
		goto	wm_Y
;
		movf	c_tmp,0
		xorlw	H'3C'
		btfsc	STATUS,Z
		goto	wm_U
;
		movf	c_tmp,0
		xorlw	H'43'
		btfsc	STATUS,Z
		goto	wm_I
;
		movf	c_tmp,0
		xorlw	H'44'
		btfsc	STATUS,Z
		goto	wm_O
;
		movf	c_tmp,0
		xorlw	H'4D'
		btfsc	STATUS,Z
		goto	wm_P
;
		movf	c_tmp,0
		xorlw	H'16'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'1E'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'26'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'25'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'2E'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'36'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'3D'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'3E'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'46'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'45'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'1C'
		btfsc	STATUS,Z
		goto	wm_A
;
		movf	c_tmp,0
		xorlw	H'1B'
		btfsc	STATUS,Z
		goto	wm_S
;
		movf	c_tmp,0
		xorlw	H'23'
		btfsc	STATUS,Z
		goto	wm_D
;
		movf	c_tmp,0
		xorlw	H'2B'
		btfsc	STATUS,Z
		goto	wm_F
;
		movf	c_tmp,0
		xorlw	H'34'
		btfsc	STATUS,Z
		goto	wm_G
;
		movf	c_tmp,0
		xorlw	H'33'
		btfsc	STATUS,Z
		goto	wm_H
;
		movf	c_tmp,0
		xorlw	H'3B'
		btfsc	STATUS,Z
		goto	wm_J
;
		movf	c_tmp,0
		xorlw	H'42'
		btfsc	STATUS,Z
		goto	wm_K
;
		movf	c_tmp,0
		xorlw	H'4B'
		btfsc	STATUS,Z
		goto	wm_L
;
		movf	c_tmp,0
		xorlw	H'1A'
		btfsc	STATUS,Z
		goto	wm_Z
;
		movf	c_tmp,0
		xorlw	H'22'
		btfsc	STATUS,Z
		goto	wm_X
;
		movf	c_tmp,0
		xorlw	H'21'
		btfsc	STATUS,Z
		goto	wm_C
;
		movf	c_tmp,0
		xorlw	H'2A'
		btfsc	STATUS,Z
		goto	wm_V
;
		movf	c_tmp,0
		xorlw	H'32'
		btfsc	STATUS,Z
		goto	wm_B
;
		movf	c_tmp,0
		xorlw	H'31'
		btfsc	STATUS,Z
		goto	wm_N
;
		movf	c_tmp,0
		xorlw	H'3A'
		btfsc	STATUS,Z
		goto	wm_M
;
		movf	c_tmp,0
		xorlw	H'29'
		btfsc	STATUS,Z
		goto	wm_space
;
		movf	c_tmp,0
		xorlw	H'6C'
		btfsc	STATUS,Z
		goto	wm_7
;
		movf	c_tmp,0
		xorlw	H'75'
		btfsc	STATUS,Z
		goto	wm_8
;
		movf	c_tmp,0
		xorlw	H'7D'
		btfsc	STATUS,Z
		goto	wm_9
;
		movf	c_tmp,0
		xorlw	H'6B'
		btfsc	STATUS,Z
		goto	wm_4
;
		movf	c_tmp,0
		xorlw	H'73'
		btfsc	STATUS,Z
		goto	wm_5
;
		movf	c_tmp,0
		xorlw	H'74'
		btfsc	STATUS,Z
		goto	wm_6
;
		movf	c_tmp,0
		xorlw	H'69'
		btfsc	STATUS,Z
		goto	wm_1
;
		movf	c_tmp,0
		xorlw	H'72'
		btfsc	STATUS,Z
		goto	wm_2
;
		movf	c_tmp,0
		xorlw	H'7A'
		btfsc	STATUS,Z
		goto	wm_3
;
		movf	c_tmp,0
		xorlw	H'70'
		btfsc	STATUS,Z
		goto	wm_0
;
		movf	c_tmp,0
		xorlw	H'71'
		btfsc	STATUS,Z
		goto	wm_bodka
;
		movf	c_tmp,0
		xorlw	H'79'
		btfsc	STATUS,Z
		goto	wm_plus
;
		movf	c_tmp,0
		xorlw	H'7B'
		btfsc	STATUS,Z
		goto	wm_minus
;
		movf	c_tmp,0
		xorlw	H'7C'
		btfsc	STATUS,Z
		goto	wm_hvizdicka

		movf	c_tmp,0
		xorlw	H'76'			;ESC
		btfsc	STATUS,Z
;!!!!!!		goto	main_s


;not known character - either transmission error or not decoded... - do nothing...
		decf	disp_pos,1
		
		return


		return


;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
w_release	call	init2

		movlw	B'10000000'		;row: 1 / col: 0
		call	send_comm

		call	wm_I
		call	wm_N
		call	wm_A
		call	wm_space
		call	wm_V
		call	wm_A
		call	wm_space
		call	wm_minus

		btfss	PORTC,D'003'
		goto	w_relaese2		;2-lines LCD

		movlw	B'11000000'		;row: 1 / col: 9
		call	send_comm

		call	wm_space
		call	wm_v
		call	wm_2
		call	wm_bodka
		call	wm_1
		call	wm_2
		call	wm_space
		call	wm_space

		return
;------------------------------------------------
w_relaese2	call	wm_space
		call	wm_v
		call	wm_2
		call	wm_bodka
		call	wm_1
		call	wm_2
		call	wm_space
		call	wm_space

		movlw	B'11000000'		;row: 2 / col: 0
		call	send_comm

		call	wm_1
		call	wm_5
		call	wm_bodka
		call	wm_0
		call	wm_2
		call	wm_bodka
		call	wm_2
		call	wm_0
		call	wm_1
		call	wm_5

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_bin7	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		call	wm_x

		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------

;-------------------------------------------------------------------------------
write_bin	movlw	'0'
		btfsc	kbd_buffer,D'007'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'006'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'005'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'004'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'003'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'002'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'001'
		movlw	'1'
		call	send_data

		movlw	'0'
		btfsc	kbd_buffer,D'000'
		movlw	'1'
		call	send_data

		return

;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_dec	movlw	'0'
		movwf	v_tst1
		movwf	v_tst2
		movwf	v_tst3

conv_next	movf	kbd_buffer,0
		btfsc	STATUS,Z
		goto	tst_conv_f

		incf	v_tst3,1
		movf	v_tst3,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn

		movlw	'0'
		movwf	v_tst3
		incf	v_tst2,1
		movf	v_tst2,0
		xorlw	':'
		btfss	STATUS,Z
		goto	wdcn

		movlw	'0'
		movwf	v_tst2
		incf	v_tst1,1

wdcn		decf	kbd_buffer,1
		goto	conv_next
;-------------------------------------------------
tst_conv_f	movf	v_tst1,0
		xorlw	'0'
		btfss	STATUS,Z
		goto	tst_conv_f3
		movlw	' '
		call	send_data
		goto	tst_conv_f2
tst_conv_f3	movf	v_tst1,0
		call	send_data
tst_conv_f2	movf	v_tst2,0
		call	send_data
		movf	v_tst3,0
		call	send_data
		return
;-------------------------------------------------------------------------------
;-------------------------------------------------------------------------------
write_hex	nop

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'00'
		btfsc	STATUS,Z
		call	wm_0

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'10'
		btfsc	STATUS,Z
		call	wm_1

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'20'
		btfsc	STATUS,Z
		call	wm_2

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'30'
		btfsc	STATUS,Z
		call	wm_3

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'40'
		btfsc	STATUS,Z
		call	wm_4

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'50'
		btfsc	STATUS,Z
		call	wm_5

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'60'
		btfsc	STATUS,Z
		call	wm_6

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'70'
		btfsc	STATUS,Z
		call	wm_7

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'80'
		btfsc	STATUS,Z
		call	wm_8

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'90'
		btfsc	STATUS,Z
		call	wm_9

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'A0'
		btfsc	STATUS,Z
		call	wm_A

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'B0'
		btfsc	STATUS,Z
		call	wm_B

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'C0'
		btfsc	STATUS,Z
		call	wm_C

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'D0'
		btfsc	STATUS,Z
		call	wm_D

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'E0'
		btfsc	STATUS,Z
		call	wm_E

		movf	kbd_buffer,0
		andlw	B'11110000'
		xorlw	H'F0'
		btfsc	STATUS,Z
		call	wm_F
;
		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'00'
		btfsc	STATUS,Z
		goto	wm_0

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'01'
		btfsc	STATUS,Z
		goto	wm_1

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'02'
		btfsc	STATUS,Z
		goto	wm_2

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'03'
		btfsc	STATUS,Z
		goto	wm_3

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'04'
		btfsc	STATUS,Z
		goto	wm_4

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'05'
		btfsc	STATUS,Z
		goto	wm_5

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'06'
		btfsc	STATUS,Z
		goto	wm_6

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'07'
		btfsc	STATUS,Z
		goto	wm_7

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'08'
		btfsc	STATUS,Z
		goto	wm_8

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'09'
		btfsc	STATUS,Z
		goto	wm_9

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0A'
		btfsc	STATUS,Z
		goto	wm_A

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0B'
		btfsc	STATUS,Z
		goto	wm_B

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0C'
		btfsc	STATUS,Z
		goto	wm_C

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0D'
		btfsc	STATUS,Z
		goto	wm_D

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0E'
		btfsc	STATUS,Z
		goto	wm_E

		movf	kbd_buffer,0
		andlw	B'00001111'
		xorlw	H'0F'
		btfsc	STATUS,Z
		goto	wm_F


		return

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;-----------------------------------------------------------------------------
i2c_start	bsf	BSR,D'002'		;select register BANK 4
		bsf	SSP1CON2, SEN
		bcf	BSR,D'002'		;select register BANK 0
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_stop	bsf	BSR,D'002'
		bsf	SSP1CON2, PEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_ack		bsf	BSR,D'002'
		bcf	SSP1CON2, ACKDT		;ACK
		bsf	SSP1CON2, ACKEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_not_ack	bsf	BSR,D'002'
		bsf	SSP1CON2, ACKDT		;NOT ACK
		bsf	SSP1CON2, ACKEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
;output: received byte is in W register
;
i2c_receive	bsf	BSR,D'002'
		bsf	SSP1CON2, RCEN
		bcf	BSR,D'002'
		call	d55
		bsf	BSR,D'002'
		movf	SSP1BUF,0
		bcf	BSR,D'002'
		return
;-----------------------------------------------------------------------------
;input: transmitted byte have to be in W register
;
i2c_send	bsf	BSR,D'002'
		movwf	SSP1BUF
		bcf	BSR,D'002'
		call	d55			;wait a while
		return
;-----------------------------------------------------------------------------
i2c_restart	bsf	BSR,D'002'
		bsf	SSP1CON2, RSEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------



;-------------------------------------------------------------------------------
d55		movlw	D'255'
		movwf	TIMER2
d55_loop	decfsz	TIMER2,F
		goto	d55_loop
		return

;----------------------------------------------------------------------------
dr01s		MOVLW   D'010'		;original value was 010
		MOVWF   TIMER2
DEL_LOOP3_1s	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP3_2s	DECFSZ  TIMER1,F
                GOTO    DEL_LOOP3_2s
                DECFSZ  TIMER2,F
                GOTO    DEL_LOOP3_1s
		RETURN


;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;I2C
;set up I2C BUS for communication as a master
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
i2c_init	movlw	H'04'
		movwf	BSR		;select BANK 4

		movlw	B'00101000'		;see Datasheet Page 277
;			bit 3-0 SSPM<3:0>: Synchronous Serial Port Mode Select bits
;			1000 = I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
		movwf	SSP1CON1

;set-up I2C speed
		movlw	B'00111111'
		movwf	SSP1ADD
		clrf	SSP1STAT

		clrf	BSR		;select BANK 0

		return


;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------

;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------
tc74_init	call	i2c_start

		movf	v_i2c_add,0
		andlw	B'11111110'		;write
		call	i2c_send		;addressing the device

		movlw	D'001'
		call	i2c_send		;command = RWCR

		movlw	D'000'
		call	i2c_send		;normal operation

		call	i2c_stop

		return
;-----------------------------------------------------------------------------------------

tc74_read	call	i2c_start

		movf	v_i2c_add,0
		andlw	B'11111110'		;write
		call	i2c_send		;addressing the device

		movlw	D'000'
		call	i2c_send		;command = RTR

		call	i2c_start

		movf	v_i2c_add,0
		iorlw	B'00000001'		;read
		call	i2c_send		;addressing the device

		call	i2c_receive		;received byte
		movwf	v_temp1
		call	i2c_not_ack

		call	i2c_stop

;------------------------------------------------------
;temperature correction - depends on the sensor
		incf	v_temp1,1
		incf	v_temp1,1
		incf	v_temp1,1
;------------------------------------------------------

		return
;-----------------------------------------------------------------------------------------
;-----------------------------------------------------------------------------------------

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx


		END